#!/bin/sh

LOG_FILE="/dnake/bin/firmware_flash_log.txt"
REBOOT_PLACE_HOLDER="/dnake/data/reboot_place_holder.txt"

SUCCESS=0
STANDARD_FAILURE=1
CRITICAL_SECTION_FAILURE=2

FILES_MATCH=0
FILES_NO_MATCH=1

upgrade_package_root=""
report_progress_enabled=0
reboot_required=0

current_model_not_set=0

#
# Logs to file and logcat at Info level.
#
logInfo()
{
    log -p i -t "Upgrade script" $1
    echo $1 >> ${LOG_FILE}
}

#
# Logs to file and logcat at Warning level.
#
logWarning()
{
    log -p w -t "Upgrade script" $1
    echo $1 >> ${LOG_FILE}
}

#
# Logs to file and logcat at Error level.
#
logError()
{
    log -p e -t "Upgrade script" $1
    echo $1 >> ${LOG_FILE}
}

#
# Reports progress to upgrade daemon process via parent pid.
#
reportProgress()
{
    # Send custom signal to upgrade daemon
    if [ ${report_progress_enabled} -eq 1 ]
    then
        kill -USR1 ${PPID}
        if [ $? -ne ${SUCCESS} ]
        then
            logWarning "Failed to report progress"
        fi
    fi
}

#
# Checks if file count in destination directory is valid.
# File count must match or be greater than file count in source directory.
#
check_file_count()
{
    local src_file_count=0
    local dest_file_count=0

    local file_count=0

    count_files_in_dir $1
    src_file_count=${file_count}

    file_count=0

    count_files_in_dir $2
    dest_file_count=${file_count}

    if [ ${dest_file_count} -lt ${src_file_count} ]
    then
        logError "Destination file count ($dest_file_count) less than source file count ($src_file_count)"
        logError "Aborting firmware flashing process!"
        exit 1
    fi
}

#
# Recursively count all files in directory (including sub-directories).
#
count_files_in_dir()
{
    for item in $1/*
    do
        if [ -f "$item" ]
        then
            file_count=$((file_count + 1))
        elif [ -d "$item" ]
        then
            count_files_in_dir ${item}
        fi
    done
}

#
# Compares MD5 hash of source and destination files.
#
compare_files()
{
    if [ -e $2 ]
    then
        # Get md5sum for each file
        src_md5_result="$(busybox md5sum $1 | busybox cut -d ' ' -f1)"
        dest_md5_result="$(busybox md5sum $2 | busybox cut -d ' ' -f1)"

        # Results will contain full path - pass to "cut" to split on spaces and compare first fields
        if [[ ${src_md5_result} != ${dest_md5_result} ]]
        then
            # Files are different
            return ${FILES_NO_MATCH}
        else
            # Files are the same
            return ${FILES_MATCH}
        fi
    else
        # Destination file does not exist
        return ${FILES_NO_MATCH}
    fi
}

#
# Disables system packages using Android's Package Manager.
#
disable_system_package()
{
    # Check if package is enabled - if so, disable
    system_ui_package="$(pm list packages -e $1)"
    # Do wildcard match as pm list packages prepends "package:" to result
    if [[ ${system_ui_package} == *$1* ]]
    then
        # Disable system ui
        logInfo "Disabling package ${system_ui_package}"
        pm disable $1
    fi
}

#
# Upgrades binary files including web interface files.
#
upgrade_binary_files()
{
    logInfo "Installing bin files..."

	# Only copy over build.prop if it has not been previously set up as this file controls model type/product
	if [ ${current_model_not_set} -eq 0 ]
	then
		for entry in `ls ${upgrade_package_root}/dnake/bin`
		do
			if [ ${entry} != "build.prop" ]
			then
				cp -rf ${upgrade_package_root}/dnake/bin/${entry} /dnake/bin
				if [ $? -ne ${SUCCESS} ]
				then
					logError "Failed to copy ${entry} to bin folder"
					return ${STANDARD_FAILURE}
				fi
			fi
		done
	else
		cp -rf ${upgrade_package_root}/dnake/bin/* /dnake/bin/
		if [ $? -ne ${SUCCESS} ]
		then
			logError "Failed to copy bin folder"
			return ${STANDARD_FAILURE}
		fi
	fi

    chmod -R 0777 /dnake/bin/*
    if [ $? -ne ${SUCCESS} ]
    then
        logError "Failed to chmod bin folder"
        return ${STANDARD_FAILURE}
    fi

    chmod 0644 /dnake/bin/*.bak

    logInfo "...done!"
}

#
# Upgrades config files and resets ringtones
#
upgrade_project_files()
{
    logInfo "Installing project files..."

    mkdir -p /dnake/cfg/system
    mkdir -p /dnake/data/system
	
	cp -rf ${upgrade_package_root}/dnake/cfg/* /dnake/cfg
	if [ $? -ne ${SUCCESS} ]
	then
		logError "Failed to copy cfg folder"
		return ${STANDARD_FAILURE}
	fi
	
    check_file_count ${upgrade_package_root}/dnake/cfg /dnake/cfg
		
	cp -rf ${upgrade_package_root}/dnake/data/system /dnake/data/
	if [ $? -ne ${SUCCESS} ]
	then
		logError "Failed to copy data folder"
		return ${STANDARD_FAILURE}
	fi
	
    check_file_count ${upgrade_package_root}/dnake/data /dnake/data
	
	rm -rf /dnake/bin/prompt
	rm -rf /dnake/bin/ringtone

    logInfo "...done!"
}

#
# Upgrades system settings including setting correct time zones, NTP, default ringtone and screen brightness.
#
upgrade_system_settings()
{
    logInfo "Configuring system settings..."

    # Set ADB to launch in tcp mode for remote debugging
    setprop persist.adb.tcp.port 5555

    # Populate ntp server with default if not already set
    local ntp_server=$(settings get global ntp_server)
    if [ ${ntp_server} == "null" ] || [ ${ntp_server} == "NULL" ] || [ ${ntp_server} == "" ]
    then
        settings put global ntp_server 2.android.pool.ntp.org
    fi

    logInfo "...done!"
}

#
# Upgrades critical system files.
# Since there is a risk of unexpected system crashes when modifying these files, files are only modified if needed.
#
upgrade_system_files()
{
    #------------------------------------------------------------
    #mount -o remount,rw /system

    #cp -f ${upgrade_package_root}/system/vendor/modules/* /system/vendor/modules/
    #chmod 0644 /system/vendor/modules/*.ko

    logInfo "Installing critical system files..."

    # Start copying over system files ONLY if they are different to incoming source files
    for item in ${upgrade_package_root}/system/app/*
    do
        # Only attempt copy if one or more files exist
        [ ! -e ${item} ] && continue

        # Strip out path name before passing file name as 2nd parameter to compare_files()
        file="$(busybox basename "$item")"
        compare_files ${item} /system/app/${file}
        if [ $? -ne ${FILES_MATCH} ]
        then
            logInfo "${file} source and destination files do not match - copying over"
            rm -f /system/app/${file}
            cp -f ${item} /system/app/${file}
            if [ $? -ne ${SUCCESS} ]
            then
                logError "Failed to copy ${file} to system folder"
                return ${CRITICAL_SECTION_FAILURE}
            fi
        fi
    done

    chmod 0644 /system/app/*.apk
	
    compare_files ${upgrade_package_root}/system/etc/bootanimation/bootanimation.zip /system/media/bootanimation.zip
    if [ $? -ne ${FILES_MATCH} ]
    then
        logInfo "Bootanimation.zip source and destination file sizes do not match - copying over"
        cp -f ${upgrade_package_root}/system/etc/bootanimation/bootanimation.zip /system/media/bootanimation.zip
        if [ $? -ne ${SUCCESS} ]
        then
            logError "Failed to copy bootanimation zip"
            return ${CRITICAL_SECTION_FAILURE}
        fi
    fi

    chmod 0644 /system/media/bootanimation.zip
    if [ $? -ne ${SUCCESS} ]
    then
        logError "Failed to chmod bootanimation zip"
        return ${STANDARD_FAILURE}
    fi
    
    for item in ${upgrade_package_root}/system/bin/*
    do
        # Only attempt copy if one or more files exist
        [ ! -e ${item} ] && continue

        # Strip out path name before passing file name as 2nd parameter to compare_files()
        file="$(busybox basename "$item")"
        compare_files ${item} /system/bin/${file}
        if [ $? -ne ${FILES_MATCH} ]
        then
            logInfo "${file} source and destination files do not match - copying over"
            cp -f ${item} /system/bin/${file}
            if [ $? -ne ${SUCCESS} ]
            then
                logError "Failed to copy ${file} to system folder"
                return ${CRITICAL_SECTION_FAILURE}
            fi
        fi
    done

    chmod 0755 /system/bin/*
	
    for item in ${upgrade_package_root}/system/framework/*
    do
        # Strip out path name before passing file name as 2nd parameter to compare_files()
        file="$(busybox basename "$item")"
        compare_files ${item} /system/framework/${file}
        if [ $? -ne ${FILES_MATCH} ]
        then
            logInfo "${file} source and destination files do not match - copying over"
            cp -f ${item} /system/framework/${file}
            if [ $? -ne ${SUCCESS} ]
            then
                logError "Failed to copy ${file} to system folder"
                return ${CRITICAL_SECTION_FAILURE}
            fi
        fi
    done

    chmod 0644 /system/framework/*
	
    for item in ${upgrade_package_root}/system/lib/*.so
    do
        # Only attempt copy if one or more files exist
        [ ! -e ${item} ] && continue

        # Strip out path name before passing file name as 2nd parameter to compare_files()
        file="$(busybox basename "$item")"
        compare_files ${item} /system/lib/${file}
        if [ $? -ne ${FILES_MATCH} ]
        then
            logInfo "${file} source and destination file sizes do not match - copying over"
            cp -f ${item} /system/lib/${file}
            if [ $? -ne ${SUCCESS} ]
            then
                logError "Failed to copy ${file} to libs folder"
                return ${CRITICAL_SECTION_FAILURE}
            fi
        fi
    done

    chmod 0644 /system/lib/*.so

    logInfo "...done!"
}

#
# Starts upgrade process.
# Currently composes of 4 main stages.
#
start_upgrade_process()
{
    logInfo "Firmware flashing process started!"

    upgrade_system_settings
    if [ $? -ne ${SUCCESS} ]
    then
        return ${STANDARD_FAILURE}
    fi

    reportProgress

    upgrade_project_files
    if [ $? -ne ${SUCCESS} ]
    then
        return ${STANDARD_FAILURE}
    fi

    reportProgress

    upgrade_binary_files
    if [ $? -ne ${SUCCESS} ]
    then
        return ${STANDARD_FAILURE}
    fi

    reportProgress

    upgrade_system_files
    if [ $? -ne ${SUCCESS} ]
    then
        return ${CRITICAL_SECTION_FAILURE}
    fi

    reportProgress

    logInfo "Firmware flashing process succeeded!"
}

#
# Checks argument validity
#
check_arguments()
{
    if [[ $# -eq 0 ]]
    then
        logError "No command arguments provided"
        exit 1
    fi
	
	while [[ $# -gt 0 ]]
	do
		key="$1"
		echo $1

		case ${key} in
			-i|--install)
                upgrade_package_root=$2
                logInfo "Upgrade package = ${upgrade_package_root}"
				shift # past argument
				shift # past value
				;;
            -r | --report)
                report_progress_enabled=1
                logInfo "Reporting enabled"
				shift # past argument
				shift # past value
				;;
            -ota)
                ota_upgrade_enabled=1
                logInfo "OTA upgrade enabled"
				shift # past argument
				shift # past value
				;;
			*)
				# Check for legacy command first
                if [[ $1 = *"udata"* ]]
                then
                    upgrade_package_root=$1
                    logInfo "Upgrade package = ${upgrade_package_root}"
					break
                else
                    logError "Unknown command received - ($1)"
                    exit 1
                fi
                ;;
		esac
	done
}

#
# Checks validity of incoming firmware file
#
check_firmware_validity()
{
	local current_model_search=$(grep "ro.product.model" /system/build.prop)
	local current_model=${current_model_search#*=}

	local incoming_model_search=$(grep "ro.product.model" ${upgrade_package_root}/dnake/bin/build.prop)
	local incoming_model=${incoming_model_search#*=}
	
	logInfo "Firmware validaty check - current model = ${current_model}, incoming model = ${incoming_model}"
	if [ ${current_model} != "ORC-4T" ] && [ ${current_model} != "ORC-7T" ] && [ ${current_model} != "ORC-7TF" ] && [ ${current_model} != "ORC-10T" ] \
		&& [ ${current_model} != "ORC-E2v2" ] && [ ${current_model} != "ORC-E3v2" ] && [ ${current_model} != "ORC-E5" ]
	then
		logInfo "Current model not set - assuming firmware is correct for model. Proceeding with flashing process"
		current_model_not_set=1
	elif [ ${current_model} != ${incoming_model} ]
	then
		logError "Current model does not match incoming model, incorrect firmware detected! Aborting flashing process!"
		exit 1
	else
		logInfo "Current model matches incoming model - proceeding with flashing process"
	fi
}

mount -o remount,rw /dnake/bin

rm -f ${LOG_FILE}
rm -f ${REBOOT_PLACE_HOLDER}

touch ${LOG_FILE}
timestamp=$(date -u +"%d-%m-%y:%T")
logInfo "Firmware flashing process starting at ${timestamp}"

check_arguments $@
check_firmware_validity
start_upgrade_process

# Store result of upgrade process and check
RESULT=$?
if [ ${RESULT} -eq ${STANDARD_FAILURE} ]
then
    # Standard failure occurred - try one more time
    logInfo "Retrying flashing process"
    start_upgrade_process
    if [ $? -ne ${SUCCESS} ]
    then
        # Failed again - abort
        logError "Aborting firmware flashing process!"
        exit 1
    else
        echo 'true' > ${REBOOT_PLACE_HOLDER}
    fi
elif [ ${RESULT} -eq ${CRITICAL_SECTION_FAILURE} ]
then
    # Failure occurred during "critical" section - abort
    logError "Critical failure - Aborting firmware flashing process!"
    exit 1
else
    echo 'true' > ${REBOOT_PLACE_HOLDER}
fi

if [ ${reboot_required} -eq 1 ]
then
    reboot
fi
